#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <portLib.h>
#include "h2initGlob.h"
#include <h2timerLib.h>

#include "server/demoPosterLib.h"
#include "demoConst.h"   

static Window win;
static Display *dpy;
static GC gc;
static double pos=0.0;

#define STD_EVENT_MASK (ButtonPressMask | KeyPressMask | \
			StructureNotifyMask | ExposureMask | ButtonReleaseMask | \
                        ButtonMotionMask )

double conv_m_pix(double longueur)
{
  /* 1 metre est represente par scale pixels */
  int scale=(300-20)/DEMO_MACHINE_LENGTH;
  return longueur*scale;
}

void draw_mobile (double pos,int x_orig,int y_orig)
{
  int larg_mobile=10;
  int x,y,xx,yy;
  int pos_translate=conv_m_pix(pos+DEMO_MACHINE_LENGTH/2);

  /* On dessine d'abord le mobile */
  x=x_orig+pos_translate-larg_mobile/2;
  y=y_orig-larg_mobile/2;
  XDrawRectangle(dpy, win, gc, x, y,larg_mobile, larg_mobile);

  x=x_orig+pos_translate-2;
  xx=x_orig+pos_translate+2;
  XDrawLine (dpy, win, gc, x, y_orig, xx, y_orig);
  x=x_orig+pos_translate;
  y=y_orig+2;
  yy=y_orig-2;
  XDrawLine (dpy, win, gc, x, y, x, yy);
  
  /* Puis le rail */
  if (pos_translate>larg_mobile/2) {
    x=x_orig+pos_translate-larg_mobile/2;
    XDrawLine (dpy, win, gc, x_orig, y_orig, x, y_orig);
  }
   if (pos_translate<conv_m_pix(DEMO_MACHINE_LENGTH)-larg_mobile/2) {
    x=x_orig+pos_translate+larg_mobile/2;
    xx=x_orig+conv_m_pix(DEMO_MACHINE_LENGTH);
    XDrawLine (dpy, win, gc, x, y_orig, xx, y_orig);
  }
}

int
main(int argc, char **argv)
{
  char *dpy_name=NULL;
  int screen;
  XWindowAttributes win_att;
  int window_x = 300;
  int window_y = 70;
  int window_width  = 300;
  int window_height = 70;
  int border_width = 5;
  XEvent event;
  XSizeHints   size_hints;
  int end = 0;
  int count = 0;
  H2TIMER_ID timer;
  DEMO_STATE_STR etat;

  dpy = XOpenDisplay(dpy_name);
  if ( dpy == NULL ) {
    fprintf(stderr, "Error Open Display!!\n");
    exit(-1);
  }

  screen = DefaultScreen(dpy);
  XGetWindowAttributes(dpy, RootWindow(dpy, screen), &win_att);

  win = XCreateSimpleWindow(dpy, RootWindow(dpy, screen),
			    window_x, window_y, window_width, window_height,
			    border_width,    
			    BlackPixel(dpy, screen),
			    WhitePixel(dpy, screen));
  size_hints.flags = PPosition | PSize | PMinSize;
  size_hints.x = window_x;
  size_hints.y = window_y;
  size_hints.width  = window_width;
  size_hints.height = window_height;
  size_hints.min_width  = 300 + 2 * border_width;
  size_hints.min_height = 70 + 2 * border_width;
  XSetStandardProperties(dpy, win, "DEMO2D", "DEMO2D",
			 None, argv, argc, &size_hints); 

  XSelectInput (dpy, win, STD_EVENT_MASK);
  XMapWindow(dpy, win);

  gc = XCreateGC(dpy, win, 0, NULL);
  XSetForeground(dpy, gc, BlackPixel(dpy, screen));


  /* Defini la valeur d'un tic (en nombre de tic par seconde */
  h2initGlob(20);

  timer = h2timerAlloc ();
  if (!timer) {
    fprintf (stderr, "Cannot alloc timer!\n");
    XDestroyWindow(dpy, win);
    XCloseDisplay(dpy);
    exit (-1);
  }
  /* Start timer */
  h2timerStart(timer, 2, 1);
    if (demoPosterInit () != OK) {
    fprintf (stderr, "Cannot Init demo poster!\n");
    XDestroyWindow(dpy, win);
    XCloseDisplay(dpy);
    exit (-2);
  }

  /* On trace demo une fois au depart */
  XClearArea(dpy,win,0,0,0,0,1);
  draw_mobile(pos,10,window_height/2);

#define STEP (5.)

  while(!end) {
    Window window;
    int xw, yw;
    
    if (h2timerPause(timer) != OK) {
      fprintf (stderr, "Cannot Pause timer!\n");
      end = 1;
      break;
    }
    
    /* lecture de la position courante dans le poster */
    if (demoMobilestatePosterRead(&etat)!=OK) {
      printf("Acces au poster impossible...\n");
      end=1;
      break;
    }

    if (etat.position!=pos) {
      pos=etat.position;
      XClearArea(dpy,win,0,0,0,0,1);
      draw_mobile(pos,10,window_height/2);
    }
    
    if (XCheckWindowEvent(dpy, win, STD_EVENT_MASK, &event)) {
      /* XNextEvent(dpy, &event); */
      switch(event.type) {
      case MotionNotify:
	/* Nothing */
	break;

      case KeyPress:
	
	switch (event.xkey.keycode) {
	case 61:		/* q comme quit */
	  end = 1;
	  break;
	case 64:                /* r comme redraw */
	  XClearArea(dpy,win,0,0,0,0,1);
	  draw_mobile(pos,10,window_height/2);
	  break;
	default:
	  printf("event.xkey.keycode = %d\n", event.xkey.keycode);
	} /* switch */
	break;

      case ButtonPress:
	switch(event.xbutton.button) {
	case Button1:
	  XClearArea(dpy,win,0,0,0,0,1);
	  draw_mobile(pos,10,window_height/2);
	  break;
	case Button2:
	  end=1;
	  break;  
	case Button3:
	  printf("Aide de cette petite application:\n");
	  printf("   q ou bouton du milieu : quit\n");
	  printf("   r ou bouton gauche    : redraw\n");
	  printf("   h ou bouton droit     : help\n");
	  break;
	}	/* switch */

      }	/* switch */
    } /* if */
  } /* while */

  h2timerFree (timer);
  XDestroyWindow(dpy, win);
  XCloseDisplay(dpy);
  return(0);  
}
